--[[Author:		Cyprias 
	License:	All Rights Reserved
	Contact:	Cyprias on Curse.com or WowAce.com

	Only Curse.com and Wowace.com have permission to host this addon.
	I have not given permission for Totem Radius to be used in any addon compilation or UI pack.]]
	
-- global lookup
local pairs_ 								= pairs
local InterfaceOptionsFrame_OpenToCategory_	= InterfaceOptionsFrame_OpenToCategory
local GetCVar_ 								= GetCVar
local select_								= select
local UnitInParty_							= UnitInParty
local UnitInRaid_							= UnitInRaid
local GetTime_								= GetTime
local GetTotemInfo_							= GetTotemInfo
local select_ 								= select
local Minimap_								= Minimap
local CreateFrame_ 							= CreateFrame
local unpack_ 								= unpack
local GetCVar_ 								= GetCVar
local math_deg								= math.deg
local math_atan2							= math.atan2
local GetPlayerMapPosition_					= GetPlayerMapPosition
local GetNumPartyMembers_					= GetNumPartyMembers
local GetNumRaidMembers_					= GetNumRaidMembers
local GetPlayerMapPosition = GetPlayerMapPosition
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapZone = GetCurrentMapZone
local SetMapToCurrentZone = SetMapToCurrentZone
local SetMapZoom = SetMapZoom
local tostring = tostring
local IsInInstance = IsInInstance
local GetSpellInfo = GetSpellInfo

local folder, core = ...
TR = core --global table

-- Isolate the environment
local _G = getfenv(0)
setmetatable(TR, {__index = _G})
setfenv(1, TR)
-- local
title		= "Totem Radius"
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version
addonDir = "Interface\\AddOns\\TotemRadius\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceSerializer-3.0") 
L = LibStub("AceLocale-3.0"):GetLocale(folder, true)


--~ local AceGUI = LibStub("AceGUI-3.0")

commPrefixTotemLoc		= "TR_L"--New totem info
commPrefixRemoveTotem	= "TR_K"--Remove totem.

--~ Astrolabe = DongleStub("Astrolabe-0.4")

activeTotems = {}
totemIcon                = "Interface\\TargetingFrame\\UI-RaidTargetingIcon_" --use raid icons if we don't know the totem icon.
minimapSize = { -- size of minimap in yards.
	indoor = {
		[0] = 150,
		[1] = 120,
		[2] = 90,
		[3] = 60,
		[4] = 40,
		[5] = 25,
	},
	outdoor = {
		[0] = 233 + 1/3,
		[1] = 200,
		[2] = 166 + 2/3,
		[3] = 133 + 1/6,
		[4] = 100,
		[5] = 66 + 2/3,
	},
}

local regEvents = {
	"PLAYER_TOTEM_UPDATE",
	"MINIMAP_UPDATE_ZOOM",
	"COMBAT_LOG_EVENT_UNFILTERED",
}
db	= {}
P	= {} --db.profile

userName = ""
indoors = Minimap:GetZoom() == GetCVar_( "minimapInsideZoom" ) + 0;

defaultSettings	= {
	profile = {
		totemColour= {},
		totemOpts = {},
	},
}

function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("TR_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("tr", "MySlashProcessorFunc")
	
	
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, titleFull)
	
	Minimap_ = _G.Minimap
	
	--Reset AddonLoader's slash commands.
	 if _G.AddonLoader then
		if _G.AddonLoader.RemoveInterfaceOptions then
			_G.AddonLoader:RemoveInterfaceOptions("Totem Radius")
		end
		
		_G["SLASH_TR1"] = nil 
		_G.SlashCmdList["TR"] = nil
		_G.hash_SlashCmdList["/tr"] = nil
	 end
end

function core:OnEnable()
	P = db.profile

	for i, event in pairs_(regEvents) do 
		self:RegisterEvent(event)
	end
	
	HookSetZoom()
	
	self:RegisterComm(commPrefixTotemLoc, "IncComm")
	self:RegisterComm(commPrefixRemoveTotem, "IncComm")

	userName = GetUnitName("player")
	GetTotemNames()
	BuildPrefixUI()
end

local function HideAllTotems()
	for GUID, data in pairs_(activeTotems) do 
		Debug("Hiding icon", GUID)
		data.icon:Hide()
	end
end

function core:OnDisable()
	HideAllTotems()
end

function GetTotemNames()
	for spellID, data in pairs(totemInfo) do 
		totemInfo[spellID].name = GetSpellInfo(spellID)
	end
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(coreOpts)
end

function GetCurrentPlayerPosition()
	local x, y = GetPlayerMapPosition("player");
	if ( x <= 0 and y <= 0 ) then
--~ 		if ( WorldMapVisible ) then
--~ 			-- we know there is a visible world map, so don't cause 
--~ 			-- WORLD_MAP_UPDATE events by changing map zoom
--~ 			return;
--~ 		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition("player");
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition("player");
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				return;
			end
		end
		local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( C ~= lastCont or Z ~= lastZone ) then
			SetMapZoom(lastCont, lastZone); --set map zoom back to what it was before
		end
		return C, Z, x, y;
	end
	return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
end

function GetUnitPosition( unit, noMapChange )
	local x, y = GetPlayerMapPosition(unit);
	if ( x <= 0 and y <= 0 ) then
		if ( noMapChange ) then
			-- no valid position on the current map, and we aren't allowed
			-- to change map zoom, so return
			return;
		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition(unit);
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition(unit);
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				return;
			end
		end
		local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( C ~= lastCont or Z ~= lastZone ) then
			SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
		end
		return C, Z, x, y;
	end
	return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
end

function core:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = ...-- ***
	if eventType == "SPELL_SUMMON" then
		local spellID, spellName, spellSchool = select_(9,...)
		if totemInfo[spellID] and self:GetCurrentMap() then
--~ 			Debug(event, 1, srcName, dstName)
		
			local tC, tZ, tX, tY
			local precise = false
			local totemSlot = totemInfo[spellID].slot --We need the school to know what direction the totem spawned. (Fire's always 45deg, Earth 270deg, ect)
				
			if UnitIsUnit("player", srcName) then
				tC, tZ = GetCurrentPlayerPosition()
				if not tC then
					return
				end
				tX, tY = GetTotemCoords(totemHeadings[totemSlot])
				
				
				precise = true
			elseif P.trackOthersTotems == true and UnitInParty_(srcName) or UnitInRaid_(srcName) then
				tC, tZ, tX, tY = GetUnitPosition(srcName)
			end
--~ 			Debug(event, 2, srcName, dstName)
			if tC then
--~ 				Debug(event, 3, srcName, dstName)
				AddTotem(dstGUID, spellID, srcName, tC, tZ, tX, tY, precise, totemSlot)
				if P.broadcastTotemLocation == true and precise == true then
					BroadcastTotemLoc(dstGUID, spellID, tC, tZ, tX, tY)
				end
			end
		end
	elseif eventType == "UNIT_DIED" or eventType == "UNIT_DESTROYED" then
		if activeTotems[dstGUID] then
			RemoveTotem(dstGUID)
		end
	end
end

local GetMapInfo = GetMapInfo
function PlaceIconOnMinimap( icon, x, y )
	local tX, tY = CoordsToYardCoords(x, y)
	
	icon.tX = tX--yard coords
	icon.tY = tY

	local found, width, height = core:GetZoneSize()
	if found then
		icon.mW = width
		icon.mH = height
		local found, currentMap = core:GetCurrentMap()
		if found then
			icon.map = currentMap
			
			icon.updateTimer = 0.05
			icon:Show()
		end
	end
--~ 	iconOnUpdate(icon, 1)
end

----------------------------------------------------------
function RemoveCastersOldTotems(caster, totemSlot)		--
-- Remove last known totem for a caster's totem slot.	--
----------------------------------------------------------
	for totemGUID, data in pairs(activeTotems) do 
		if data.caster == caster and data.totemSlot == totemSlot then
--~ 			Debug("RemoveCastersOldTotems 1", "Removing", caster, totemSlot)
			RemoveTotem(totemGUID)
			return
		end
	end
end

--[[
local function iconOnHide(self)
	self:SetParent(nil)
end
]]

--------------------------------------------------------------------------------------
function AddTotem(totemGUID, spellID, caster, tC, tZ, tX, tY, precise, totemSlot)	--
-- Add a totem to our table then show it on the minimap.							--
--------------------------------------------------------------------------------------
--~ 	Debug("AddTotem 1", caster, totemGUID)
	if not totemInfo[spellID] then
		return
	end
	RemoveCastersOldTotems(caster, totemSlot)
	
	local totemName = totemInfo[spellID].name or GetSpellInfo(spellID)
	if P.totemOpts[totemName] and P.totemOpts[totemName].shown == false then
--~ 		Debug("AddTotem","User doesnt want "..tostring(totemName).." shown.")
		return
	end	
	--
	
	local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
	local minimapWidth = Minimap_:GetWidth()
	
	local range = totemInfo[spellID].range or 0
	if precise == false and range > spawnDistance then
		range = range - spawnDistance --We don't know the precise coords of the totem. So we remove 3 yards from the totem's range to make sure we'll be inside it. 
	end
	
	local size = (minimapWidth * range/radius)
	if not activeTotems[totemGUID] then
		local totemSlot = totemSlot or totemInfo[spellID].slot or 1
		
		activeTotems[totemGUID] = {
			icon	= CreateFrame_("Button",nil,Minimap_),
			caster	= caster,
			coords	= {tC, tZ, tX, tY},
			range	= range,
			precise	= precise,
			totemSlot = totemSlot,
			death = GetTime_() + totemInfo[spellID].duration,--Remove the icon at this time, just in case UNIT_DEAD doesn't see it.
			spellID = spellID,
			name = totemName,
		}
--~ 		activeTotems[totemGUID].icon:SetScript("OnHide", iconOnHide)
		
		activeTotems[totemGUID].icon.totem = activeTotems[totemGUID] --Add our totem table to the icon frame so it can be used in the OnUpdate script.
		activeTotems[totemGUID].icon.GUID = totemGUID
		activeTotems[totemGUID].icon.spellID = spellID
		
--~ 		Debug("AddTotem 2", caster, totemGUID)
		SetOnUpdateScript(activeTotems[totemGUID].icon)
		
		activeTotems[totemGUID].icon:SetWidth(P.iconSize)
		activeTotems[totemGUID].icon:SetHeight(P.iconSize)
		
		
		activeTotems[totemGUID].icon.spellTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"ARTWORK")
		activeTotems[totemGUID].icon.spellTexture:SetTexture(totemInfo[spellID].texture or totemIcon..totemSlot)
		activeTotems[totemGUID].icon.spellTexture:SetAllPoints(activeTotems[totemGUID].icon)

		if range > 0 then
			activeTotems[totemGUID].icon.solidTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"BACKGROUND")
			activeTotems[totemGUID].icon.solidTexture:SetTexture(addonDir.."texture\\Solid")
			activeTotems[totemGUID].icon.solidTexture:SetPoint("CENTER", activeTotems[totemGUID].icon);
	--~ 		activeTotems[totemGUID].icon.solidTexture:SetFrameStrata("BACKGROUND")
	--~ 		activeTotems[totemGUID].icon.solidTexture:SetAlpha(0.3)
			
			activeTotems[totemGUID].icon.outlineTexture = activeTotems[totemGUID].icon:CreateTexture(nil,"low")
			activeTotems[totemGUID].icon.outlineTexture:SetTexture(addonDir.."texture\\Outline")
			activeTotems[totemGUID].icon.outlineTexture:SetPoint("CENTER", activeTotems[totemGUID].icon);
	--~ 		activeTotems[totemGUID].icon.outlineTexture:SetFrameStrata("LOW")
			activeTotems[totemGUID].icon.outlineTexture:SetAlpha(1)
			
			activeTotems[totemGUID].icon.solidTexture:SetVertexColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, P.totemColour[totemSlot].a);
			activeTotems[totemGUID].icon.outlineTexture:SetVertexColor(0, 1, 0, 1); --first drop, colour it green.
		end
	elseif precise == true then
		--We're already tacking the totem but another user has sent us more accurate coords.
		Debug("AddTotem", tostring(caster).." has sent us more accurate coords for "..tostring(totemInfo[spellID].name))
		activeTotems[totemGUID].coords = {tC, tZ, tX, tY}
		activeTotems[totemGUID].range = range
	end
	local icon = activeTotems[totemGUID].icon

	if range > 0 then
		icon.solidTexture:SetWidth(size) -- Set these to whatever height/width is needed 
		icon.solidTexture:SetHeight(size) -- for your Texture
		
		icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
		icon.outlineTexture:SetHeight(size) -- for your Texture
	end
--~ 	icon:SetScript("OnUpdate", iconOnUpdate)

	if tC == GetCurrentMapContinent() and tZ == GetCurrentMapZone() then
		if core:GetCurrentMap() then
			PlaceIconOnMinimap(icon, tX, tY)
			icon:Show()
		end
	end
	--[[
	local result = Astrolabe:PlaceIconOnMinimap( activeTotems[totemGUID].icon, tC, tZ, tX, tY )
	]]
	
end

function HookSetZoom()--	/run TR.HookSetZoom()
	if not core:IsHooked(Minimap_, "SetZoom") then
		core:SecureHook(Minimap_, "SetZoom", "Minimap_SetZoom")
	end
end

function DropFakeTotem()--	/run TR.DropFakeTotem()
--~ 	local totems = {}
--~ 	for spellID in pairs(zone_data) do 
--~ 		table.insert(totems, #totems+1, spellID)
--~ 	end

	local found, currentMap = core:GetCurrentMap()
	Debug("DropFakeTotem", found, currentMap)
	if found then
		local tC, tZ = GetCurrentMapContinent(), GetCurrentMapZone();
		local tX, tY = GetTotemCoords(totemHeadings[math.random(4)])

		AddTotem(0, 2062, UnitName("player"), tC, tZ, tX, tY, true)
	end
end
--	/run print(GetMouseFocus():GetName())

function core:IncComm(prefix, message, distribution, sender)
	if sender ~= userName then
		if prefix == commPrefixTotemLoc then
			if P.trackOthersTotems == true then
				local success, totemGUID, spellID, tC, tZ, tX, tY = core:Deserialize(message)
		--~ 		Debug("IncComm",sender.." dropped a totem!")
				if success then
					AddTotem(totemGUID, spellID, sender, tC, tZ, tX, tY, true)
				end
			end
		elseif prefix == commPrefixRemoveTotem then
			local success, totemGUID = core:Deserialize(message)
			if success then
				if activeTotems[totemGUID] and activeTotems[totemGUID].caster == sender then
--~ 					Debug("IncComm","Removing "..sender.."'s totem.")
					RemoveTotem(totemGUID)
				end
			end
		end
	end
end

----------------------------------------------------------------------------------------------------------
function BroadcastTotemLoc(totemGUID, spellID, tC, tZ, tX, tY)											--
-- Send the precise coords of our totems so others can update the locs of our totems on their minimap.	--
----------------------------------------------------------------------------------------------------------
	if GetNumPartyMembers_() > 0 then
		local str = core:Serialize(totemGUID, spellID, tC, tZ, tX, tY)
	--~ 	Debug("BroadcastTotemLoc", str, str:len())
--~ 		core:SendCommMessage(commPrefixTotemLoc, str, "GUILD", nil, "ALERT")

		local iActive, iType = IsInInstance()
		if iActive and iType == "pvp" then
			core:SendCommMessage(commPrefixTotemLoc, str, "BATTLEGROUND", nil, "ALERT")
		else
			core:SendCommMessage(commPrefixTotemLoc, str, GetNumRaidMembers_() > 1 and "raid" or "party", nil, "ALERT")
		end
	end
end

----------------------------------------------------------------------------------
function BroadcastRemoveTotem(totemGUID)										--
-- Sends the totemGUID to group for others to remove if from their minimap.		--
-- This is needed cause UNIT_DEAD doesn't always fire if we're out of range.	--
----------------------------------------------------------------------------------
	if GetNumPartyMembers_() > 0 then
		local str = core:Serialize(totemGUID)
--~ 		core:SendCommMessage(commPrefixRemoveTotem, str, "GUILD", nil, "ALERT")

		local iActive, iType = IsInInstance()
		if iActive and iType == "pvp" then
			--We're in a battlefield. 
			core:SendCommMessage(commPrefixRemoveTotem, str, "BATTLEGROUND", nil, "BULK")
		else
			core:SendCommMessage(commPrefixRemoveTotem, str, GetNumRaidMembers_() > 1 and "raid" or "party", nil, "BULK")
		end
	end
end


function RemoveTotem(totemGUID)
	if activeTotems[totemGUID] then
		activeTotems[totemGUID].icon:Hide()
		activeTotems[totemGUID].icon:SetParent(nil)
		activeTotems[totemGUID].icon = nil
		activeTotems[totemGUID] = nil
	end
end


function SetOnUpdateScript(frame)
	frame.lastUpdate = 0
--~ 	frame.lastDist = 0
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.lastUpdate = self.lastUpdate + elapsed
--~ 		if (not self.onEdge and self.lastUpdate > 0.05) or (self.onEdge and self.lastUpdate > 1) then
		if self.lastUpdate > self.updateTimer  then

			self.lastUpdate = 0
			
--~ 			Debug("OnUpdate", totemInfo[self.spellID].name, self.totem.death - GetTime_())
			local found, currentMap = core:GetCurrentMap()
			
			if not found then
				Debug("OnUpdate", "Can't find current map!")
				RemoveTotem(self.GUID)
				return
			end
			
			if self.map ~= currentMap then
				--We've changed zones. Lets set the icon as transparent and set it to update once a second for now.
				self:SetAlpha(0)
				self.updateTimer = 1
				return
			else
				self:SetAlpha(1)
			end
			
			if GetTime_() >= self.totem.death then
				--Totem should have died due to old age by now. Remove it.
				RemoveTotem(self.GUID)
				return
			end

			local map = Minimap
--~ 			local minimapZoom = map:GetZoom()
		
			local tX, tY = self.tX, self.tY

			local sX, sY = GetPlayerMapPosition_("player")
		
			local sX = self.mW * sX
			local sY = self.mH * sY
		
			local dist, xDist, yDist = ComputeDistance(sX, sY, tX, tY)
			
			if dist > P.minimapIconRange then
				RemoveTotem(self.GUID)
				return
			elseif dist < self.totem.range then --hasBuff and 
				if self.totem.range > 0 then
					self.outlineTexture:SetVertexColor(0, 1, 0);
				end
			else
				if self.totem.range > 0 then
					if dist > self.totem.range then
						self.outlineTexture:SetVertexColor(1, 0, 0);
					else
						self.outlineTexture:SetVertexColor(0, 1, 0);
					end
				end
			end

			local onEdge, xDist, xScale, yDist, yScale = GetMinimapCoordOffset(self, dist, xDist, yDist)
			if onEdge then
				if self.totem.range > 0 then
					self.solidTexture:SetAlpha(0)
					self.outlineTexture:SetAlpha(0)
				end
				self.updateTimer = 0.25 --Update the position 4x a second.
			else
				if self.totem.range > 0 then
					self.solidTexture:SetAlpha(P.totemColour[self.totem.totemSlot].a)
					self.outlineTexture:SetAlpha(1)
				end
				self.updateTimer = 0.05 --Update the position 20x a second. This appears fluid above 20FPS.
			end
			
			self:ClearAllPoints();
			self:SetPoint("CENTER", map, "CENTER", xDist/xScale, -yDist/yScale);
			
		end
	end)
end
--	/run TR.DropFakeTotem()

local lastZoom = 0
local ignoreZoom = false
function core:MINIMAP_UPDATE_ZOOM(event, ...)
--~ 	Debug(event, "a", radius, minimapWidth)
	if ignoreZoom == true then --Keeps us from entering a endless loop due to SetZoom(zoom>0...) later on.
		return
	end
--~ 	Debug(event, "b", radius, minimapWidth)
	local zoom = Minimap:GetZoom();
	if zoom ~= lastZoom then --Keeps us from processing the same zoom info repeatedly due to SetZome(zoom) later on. 
--~ 		Debug(event, "c", radius, minimapWidth)
		lastZoom = zoom
		if ( GetCVar_( "minimapZoom" ) == GetCVar_( "minimapInsideZoom" ) ) then -- Indeterminate case
			ignoreZoom = true
 			Minimap:SetZoom( zoom > 0 and zoom - 1 or zoom + 1 ); -- Any change to make the cvars unequal
			ignoreZoom = false
		end
		indoors = Minimap:GetZoom() == GetCVar_( "minimapInsideZoom" ) + 0;
		Minimap:SetZoom(zoom);

		local range, size
		local radius = minimapSize[indoors and "indoor" or "outdoor"][Minimap_:GetZoom()]
		local minimapWidth = Minimap_:GetWidth()
		
--~ 		Debug(event, "d", radius, minimapWidth)
		
--~ 		for name, data in pairs_(myTotems) do 
		for GUID, data in pairs_(activeTotems) do 
			size = (minimapWidth * data.range/radius)
			
--~ 			Debug(event, indoors, radius, minimapWidth, size)
			if data.range > 0 then
				data.icon.solidTexture:SetWidth(size) -- Set these to whatever height/width is needed 
				data.icon.solidTexture:SetHeight(size) -- for your Texture
				
				data.icon.outlineTexture:SetWidth(size) -- Set these to whatever height/width is needed 
				data.icon.outlineTexture:SetHeight(size) -- for your Texture
			end
--~ 			Debug(event, "e", GUID, size)
		end
	end
end


function core:PLAYER_TOTEM_UPDATE(event, totemSlot)
	local haveTotem, totemName, startTime, duration = GetTotemInfo(totemSlot)
--~ 	Debug(event, totemSlot, haveTotem, totemName, startTime, duration)
	
	if startTime == 0 then
		for GUID, data in pairs_(activeTotems) do 
			if data.caster == userName and data.totemSlot == totemSlot then
--~ 				Debug(event,totemInfo[data.spellID].name)
				RemoveTotem(GUID)
				if P.broadcastTotemLocation == true then
					BroadcastRemoveTotem(GUID)
				end
				break
			end
		end
	end
end

function core:Minimap_SetZoom(self, ...)
--~ 	Debug("SetZoom", self:GetName(), ...)
	core:MINIMAP_UPDATE_ZOOM("MINIMAP_UPDATE_ZOOM")
end

function MapTest()--	/run TR.MapTest()
	local mapFile = GetMapInfo()
	Debug("MapTest", tostring(mapFile), tostring(core.zone_data[mapFile]))
end


--	/run TR.DropFakeTotem()
local f = CreateFrame("frame")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:SetScript("OnEvent", function(self, event)
--~ 	Debug(event)
	HideAllTotems()
end)
