--Globals
local pairs = pairs

-- Isolate the environment
local folder, core = ...
setfenv(1, core)
-- local

CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = L["Core"],
			type = "group",
			order = 1,
			args={}
		}
	},
}



CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= L["Enable"],
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable()
		else
			core:Disable()
		end
	end,
	get = function(info) return core:IsEnabled() end
}

defaultSettings.profile.trackOthersTotems = true
CoreOptionsTable.args.core.args.trackOthersTotems = {
	type = "toggle",	order	= 2,
	name	= L["Track other's totems"],
	desc	= L["Add other shaman's totems to your minimap."],
	set = function(info,val) 
		P.trackOthersTotems = val
	end,
	get = function(info) return P.trackOthersTotems end
}

defaultSettings.profile.broadcastTotemLocation = true
CoreOptionsTable.args.core.args.broadcastTotemLocation = {
	type = "toggle",	order	= 3,
	name	= L["Broadcast totem location"],
	desc	= L["Broadcast your totem's location to group members."],
	set = function(info,val) 
		P.broadcastTotemLocation = val
	end,
	get = function(info) return P.broadcastTotemLocation end
}

defaultSettings.profile.totemColour[1]	= {r=1, g=0, b=0, a=0.15} --red
CoreOptionsTable.args.core.args.fireColour = {
	name = L["Fire Colour"],	order	= 10,
	desc = L["Colour of the circle around the totem"],
	type = "color",
	hasAlpha	= true,
	set = function(info,r,g,b,a) 
		P.totemColour[1].r = r
		P.totemColour[1].g = g
		P.totemColour[1].b = b
		P.totemColour[1].a = a
		
		for GUID, data in pairs(activeTotems) do 
			if data.totemSlot == 1 then
				data.icon.solidTexture:SetVertexColor(r, g, b, a);
			end
		end
--~ 		print(Round(r,2), Round(g,2), Round(b,2), Round(a,2))
	end,
	get = function(info) return P.totemColour[1].r, P.totemColour[1].g, P.totemColour[1].b, P.totemColour[1].a end
}

defaultSettings.profile.totemColour[2]	= {r=0, g=1, b=0, a=0.2} --green
CoreOptionsTable.args.core.args.earthColour = {
	name = L["Earth Colour"],	order	= 11,
	desc = L["Colour of the circle around the totem"],
	type = "color",
	hasAlpha	= true,
	set = function(info,r,g,b,a) 
		P.totemColour[2].r = r
		P.totemColour[2].g = g
		P.totemColour[2].b = b
		P.totemColour[2].a = a
		
		for GUID, data in pairs(activeTotems) do 
			if data.totemSlot == 2 then
				data.icon.solidTexture:SetVertexColor(r, g, b, a);
			end
		end
	end,
	get = function(info) return P.totemColour[2].r, P.totemColour[2].g, P.totemColour[2].b, P.totemColour[2].a end
}

defaultSettings.profile.totemColour[3]	= {r=0, g=1, b=1, a=0.3} --cyan
CoreOptionsTable.args.core.args.waterColour = {
	name = L["Water Colour"],	order	= 12,
	desc = L["Colour of the circle around the totem"],
	type = "color",
	hasAlpha	= true,
	set = function(info,r,g,b,a) 
		P.totemColour[3].r = r
		P.totemColour[3].g = g
		P.totemColour[3].b = b
		P.totemColour[3].a = a
		
		for GUID, data in pairs(activeTotems) do 
			if data.totemSlot == 3 then
				data.icon.solidTexture:SetVertexColor(r, g, b, a);
			end
		end
	end,
	get = function(info) return P.totemColour[3].r, P.totemColour[3].g, P.totemColour[3].b, P.totemColour[3].a end
}

defaultSettings.profile.totemColour[4]	= {r=1, g=1, b=1, a=0.4} --white
CoreOptionsTable.args.core.args.airColour = {
	name = L["Air Colour"],	order	= 13,
	desc = L["Colour of the circle around the totem"],
	type = "color",
	hasAlpha	= true,
	set = function(info,r,g,b,a) 
		P.totemColour[4].r = r
		P.totemColour[4].g = g
		P.totemColour[4].b = b
		P.totemColour[4].a = a
		
		for GUID, data in pairs(activeTotems) do 
			if data.totemSlot == 4 then
				data.icon.solidTexture:SetVertexColor(r, g, b, a);
			end
		end
	end,
	get = function(info) return P.totemColour[4].r, P.totemColour[4].g, P.totemColour[4].b, P.totemColour[4].a end
}

defaultSettings.profile.minimapIconRange = 200
CoreOptionsTable.args.core.args.minimapIconRange = {
	type	= "range",	order	= 16,
	name = L["Minimap Show Range"],
	desc = L["Hide the icon if we're beyond * yards away."],
	min		= 50,
	max		= 200,
	step	= 5,
	set = function(info,val) 
		P.minimapIconRange = val;
	end,
	get = function(info) return P.minimapIconRange end
}

defaultSettings.profile.iconSize = 12
CoreOptionsTable.args.core.args.iconSize = {
	type	= "range",	order	= 17,
	name = L["Icon Size"],
	desc = L["Size of the totem icon on the minimap. (Not the circle)"],
	min		= 4,
	max		= 32,
	step	= 4,
	set = function(info,val) 
		P.iconSize = val;
		
		for GUID, data in pairs(activeTotems) do 
			data.icon:SetWidth(val)
			data.icon:SetHeight(val)
		end
	end,
	get = function(info) return P.iconSize end
}

local table_sort = table.sort
local table_insert = table.insert
local table_getn = table.getn
--Totems page
function BuildPrefixUI()--	/script TR.BuildPrefixUI()
	CoreOptionsTable.args.totems = { --reset
		name = L["Totems"],
		type = "group",
		order = 2,
		args={}
	}
	
	
	local totems = {} --sortable list
	
	local totemName
	for spellID, data in pairs(core.totemInfo) do 
		totemName = data.name or GetSpellInfo(spellID)
		if table_getn(totems) == 0 then
			table_insert(totems, #totems+1, totemName)
		else
			for i=1, table_getn(totems) do 
				if totems[i] == totemName then
					break
				elseif i == table_getn(totems) then
					table_insert(totems, #totems+1, totemName)
				end
			end
		end
		
	end
	
	table_sort(totems, function(a,b) 
		if(a and b) then 
			return a < b
		end 
	end)
		--totemOpts
	for i=1, table_getn(totems) do 
		totemName = totems[i]
--~ 		Debug("BuildPrefixUI", i, totems[i])
		
		if P.totemOpts[totemName] == nil then
			P.totemOpts[totemName] = {shown = true} --show by default, using a table incase I need to add options later.
		end
		
		CoreOptionsTable.args.totems.args[totemName] = {
			type = "toggle",	order	= i,
			name = totemName,
			desc = L["Show %s on minimap."]:format(totemName),
			set = function(info,val) 
				local totemName = info[2]
				P.totemOpts[totemName].shown = val
				
				for totemGUID, data in pairs(activeTotems) do 
					if data.name == totemName then
						RemoveTotem(totemGUID)
					end
				end
			end,
			get = function(info) return P.totemOpts[info[2]] and P.totemOpts[info[2]].shown end
		}
		
		
	end
	
	
end
